/* 
 * Queries for testing SQL Server 2014 improvements
 * by Niko Neugebauer (http://www.nikoport.com)
 */
 use [ContosoRetailDW];

create clustered columnstore index [PK_FactOnlineSales_SalesKey] 
	on dbo.FactOnlineSales;

-- Check Segments
select segment_id, row_count, base_id, min_data_id, max_data_id
	from sys.column_store_segments
	where column_id = 1;

-- Test Query
set statistics io on 
select sum([UnitPrice]) as Sales
	from dbo.FactOnlineSales
	where OnlineSalesKey >= 27120879 and OnlineSalesKey <= 32181089;

/*
50143 logical Reads
1     physical reads 
1869  read-ahead reads 
*/

-- Create an ordered Rowstore index (1:10)
create clustered index [PK_FactOnlineSales_SalesKey] on dbo.FactOnlineSales (OnlineSalesKey)
	with (DROP_EXISTING = ON, DATA_COMPRESSION = PAGE);

-- Create a Columnstore, while dropping existing RowStore Index
create clustered columnstore index [PK_FactOnlineSales_SalesKey] on dbo.FactOnlineSales
	with (DROP_EXISTING = ON);

-- Check Segments
select segment_id, row_count, base_id, min_data_id, max_data_id
	from sys.column_store_segments
	where column_id = 1;

-- Test Query Again
set statistics io on 
select sum([UnitPrice]) as Sales
	from dbo.FactOnlineSales
	where OnlineSalesKey >= 27120879 and OnlineSalesKey <= 32181089;

-- *****************************************************************************************

create clustered index [PK_FactOnlineSales_SalesKey] on dbo.FactOnlineSales (OnlineSalesKey)
	with (DROP_EXISTING = ON, DATA_COMPRESSION = PAGE);

-- Create a Columnstore with MAXDOP = 1, to get a perfect Segment Clustering
create clustered columnstore index [PK_FactOnlineSales_SalesKey] on dbo.FactOnlineSales
	with (DROP_EXISTING = ON, MAXDOP = 1);

-- Check Segments
select segment_id, row_count, base_id, min_data_id, max_data_id
	from sys.column_store_segments
	where column_id = 1;

-- Test Query Again
set statistics io on 
select sum([UnitPrice]) as Sales
	from dbo.FactOnlineSales
	where OnlineSalesKey >= 27120879 and OnlineSalesKey <= 32181089;

/*
4671 logical Reads
1     physical reads 
15065  read-ahead reads 
*/

/*
Compare with previous results:
50143 logical Reads
1     physical reads 
1869  read-ahead reads 
*/
