/* 
 * Queries for testing SQL Server 2014 improvements
 * by Niko Neugebauer (http://www.nikoport.com)
 */

-- Clustered Columnstore Indexes Analysis
SELECT i.name, p.object_id, p.index_id, i.type_desc 
   	,sum(p.rows)/count(seg.segment_id) as 'rows'
	,sum(deleted_rows) as 'deleted rows'
	,cast(sum(p.rows)/count(seg.segment_id) / COALESCE(NULLIF(sum(deleted_rows),0),10000000000000) as Decimal(8,2)) as 'Percentage Deleted'
	,sum(seg.on_disk_size) as 'size in Bytes'
	,cast( sum(seg.on_disk_size) / 1024. / 1024. as decimal(8,3)) as 'size in MB'
	,count(distinct seg.segment_id) as 'Segments'
	,count(distinct p.partition_id) as 'Partitions'
	FROM sys.column_store_segments AS seg 
		INNER JOIN sys.partitions AS p 
			ON seg.hobt_id = p.hobt_id 
		INNER JOIN sys.indexes AS i 
			ON p.object_id = i.object_id
		INNER JOIN sys.column_store_row_groups rg
			ON i.object_id = rg.object_id
	WHERE i.type in (5, 6)
		--and i.object_id = object_id('FactOnlineSales')
	GROUP BY i.name, p.object_id, p.index_id, i.type_desc;
GO
